package com.meituan.sqt.h5Access;
import org.apache.commons.lang3.RandomStringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.URLEncoder;
import java.security.interfaces.RSAPrivateKey;
import java.text.SimpleDateFormat;
import java.util.Date;

//调用该类main方法前，需要通过com.meituan.sqt.h5Access.RsaUtil.main生成公钥和私钥
public class SqtH5Access {
    private static Logger logger = LoggerFactory.getLogger(SqtH5Access.class);
    /**
     * 获取h5免登url
     * @param url 商企通接口地址
     * @param privateKey 私钥，可通过RsaUtil.genKey()生成
     * @param appkey
     * @param entId 企业Id，商企通提供
     * @param productType
     * @param staffNo 该字段与企业员工唯一标志对应，可能为staffPhoneNo字段
     */
    public static String getH5AccessUrl(String url, String privateKey, String appkey, Integer entId, String productType, String staffNo){
        try {
            // params是待签名文本，包括所有需要签名的参数，不包括value为空的参数、signature参数
            // 另外，params中，每一个键值对按key从a-z进行排序，首字母相同，则比较下一个，以此类推
            String params =  "appKey=" + appkey + "&entId=" + entId + "&nounce=" + getNounce() + "&productType=" + productType + "&requestTime=" + getRequestTime()  + "&staffNo=" + staffNo;
            RSAPrivateKey rsaPrivateKey = RsaUtil.loadPrivateKey(privateKey);
            String signature = RsaUtil.sign(params.getBytes(), rsaPrivateKey);
            //signature参数需encode
            String h5AccessUrl = url+"?"+params + "&signature="+ URLEncoder.encode(signature,"utf-8");
            return h5AccessUrl;
        } catch (Exception e) {
            logger.error("获取h5AccessUrl失败",e);
        }
        return null;
    }
    private static String getRequestTime(){
        SimpleDateFormat formatter= new SimpleDateFormat("yyyyMMddHHmmss");
        Date now = new Date(System.currentTimeMillis());
        return formatter.format(now);
    }
    private static String getNounce(){
        return RandomStringUtils.random(36, true, true);
    }


    public static void main(String[] args) {
        //接口地址
        String url = "https://m-sqt.meituan.com/open/commonaccess/access";
        //商企通提供
        String appkey = null;
        //私钥，通过 com.meituan.sqt.h5Access.RsaUtil.main 生成
        String privateKey = null;
        // 企业Id，商企通提供
        Integer entId = null;
        // 详情见业务接口描述
        String productType = null;
        // 与企业员工唯一标志对应，可能为staffPhoneNo字段
        String staffNo = null;
        String h5AccessUrl = SqtH5Access.getH5AccessUrl(url, privateKey, appkey, entId, productType, staffNo);
        System.out.println(h5AccessUrl);
    }
}

